/***************************************************************************
 *   Copyright 2008 by Davide Bettio <davide.bettio@kdemail.net>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include <assert.h>

#include <QPainter>
#include <QEvent>
#include <QTimer>

#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <fixx11h.h>
#include <QX11Info>

#include <Plasma/Svg>
#include <Plasma/Theme>
#include <KDialog>
#include <KDebug>
#include <KDialog>

#include "keyboardLeds.h"

KeyboardLeds::KeyboardLeds(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
    m_sizedirty(true),
    numLock(0),
    capsLock(0),
    scrollLock(0)
{
    resize(size());
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
}

void KeyboardLeds::init()
{
     m_svg = new Plasma::Svg(this);
     m_svg->setImagePath("widgets/keyboardleds");
     m_svg->setContainsMultipleImages(true);

     numLockMask = getKeyMask(XK_Num_Lock);
     capsLockMask = getKeyMask(XK_Caps_Lock);
     scrollLockMask = getKeyMask(XK_Scroll_Lock);

     QTimer *timer = new QTimer(this);
     connect(timer, SIGNAL(timeout()), this, SLOT(updateLedsStatus()));
     timer->start(250);
}

KeyboardLeds::~KeyboardLeds()
{

}

Qt::Orientations KeyboardLeds::expandingDirections() const
{
    return 0;
}

QSizeF KeyboardLeds::contentSizeHint() const
{
    QSizeF sizeHint = geometry().size();

    switch (formFactor()) {
        case Plasma::Vertical:
            sizeHint.setHeight(sizeHint.width());
            break;

        case Plasma::Horizontal:
            sizeHint.setWidth(sizeHint.height() / 4);
            break;

        default:
            sizeHint.setWidth(sizeHint.height() / 4);
            break;
    }

    return sizeHint;
}

void KeyboardLeds::configAccepted()
{
    update();
}

unsigned int KeyboardLeds::getKeyMask(unsigned int key)
{
    XModifierKeymap* map = XGetModifierMapping(QX11Info::display());
    KeyCode keyCode = XKeysymToKeycode(QX11Info::display(), key);

    int keyMask = 0;

    for (int i = 0; i < 8; i++) {
        if (map->modifiermap[map->max_keypermod * i] == keyCode) {
            keyMask = 1 << i;
            break;
        }
    }

    return keyMask;
}

void KeyboardLeds::updateLedsStatus()
{
    unsigned int lmask;
    Window dummyW;
    int dummyI;

    XQueryPointer(QX11Info::display(), DefaultRootWindow(QX11Info::display()), &dummyW, &dummyW,
                  &dummyI, &dummyI, &dummyI, &dummyI, &lmask);

    bool doUpdate = false;

    if ((lmask & numLockMask) != numLock){
        numLock = (lmask & numLockMask);
        doUpdate = true;
    }

    if ((lmask & capsLockMask) != capsLock){
        capsLock = (lmask & capsLockMask);
        doUpdate = true;
    }

    if ((lmask & scrollLockMask) != scrollLock){
        scrollLock = (lmask & scrollLockMask);
        doUpdate = true;
    }

    if (doUpdate) {
        update();
    }
}

void KeyboardLeds::constraintsEvent(Plasma::Constraints constraints)
{
    if (constraints & Plasma::FormFactorConstraint || constraints & Plasma::SizeConstraint) {
        m_sizedirty = true;
    }
}

void KeyboardLeds::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &rect)
{
    Q_UNUSED(option)

    const bool horizontal = formFactor() != Plasma::Vertical;

    if (m_sizedirty) {
        m_sizedirty = false;

        m_svg->resize(
            horizontal ? QSize(rect.width() / 3, rect.height())
                       : QSize(rect.width(), rect.height() / 3)
        );
    }

    QSize s = m_svg->size();
    int x = horizontal ? s.width() : 0;
    int y = horizontal ? 0 : s.height();

    if (numLock) {
        m_svg->paint(p, QRectF(rect.x(), rect.y(), s.width(), s.height()), "led");
    }

    if (capsLock) {
        m_svg->paint(p, QRectF(rect.x() + x, rect.y() + y, s.width(), s.height()), "led");
    }

    if (scrollLock) {
        m_svg->paint(p, QRectF(rect.x() + x * 2, rect.y() + y * 2, s.width(), s.height()), "led");
    }
}

#include "keyboardLeds.moc"
